@@ -1,6 +1,6 @@
 /*
  * This file is part of the QPxTool project.
- * Copyright (C) 2005-2006 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
+ * Copyright (C) 2005-2007 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,7 +13,6 @@
  *
  */
 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,34 +23,206 @@
 #include <qpx_mmc.h>
 #include "plextor_features.h"
 
+static const char progress[] = { '-', '\\', '|', '/', 0 };
+
 #define ASDB_SAVE_DEBUG
 #define ASDB_LOAD_DEBUG
+// #define DEBUG_SECUREC
+//#define SHOW_RAW_TIME
+
+int plextor_reboot(drive_info* drive) {
+	drive->cmd[0] = 0xEE;
+	if ((drive->err=drive->cmd.transport(NONE,NULL,0) ))
+		{ sperror ("reset",drive->err); return 1; }
+	return 0;
+}
 
 int plextor_get_TLA(drive_info* drive) {
-	drive->cmd_clear();
-	drive->cmd[0] = 0xF1;
+	if (strncmp(drive->ven,"PLEXTOR ",8) || !strncmp(drive->dev,"CD-R   PREMIUM2",15)) {
+		strcpy(drive->TLA,"N/A\0");
+		return 1;
+	}
+	drive->cmd[0] = PLEXTOR_EEPROM_READ;
 	drive->cmd[8] = 0x01;
 	drive->cmd[9] = 0x00;
+	drive->cmd[10] = 0x00;
+	drive->cmd[11] = 0x00;
 	// The Plextor PX-716 does not understand this command....
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x100)))
 	{
 		//printf("Possible PX-716...\n");
-		drive->cmd_clear();
 		drive->cmd[0] = 0xF1;
 		drive->cmd[1] = 0x01;
 		drive->cmd[8] = 0x01;
 		drive->cmd[9] = 0x00;
+		drive->cmd[10] = 0x00;
+		drive->cmd[11] = 0x00;
 		if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x100) ))
-			{strcpy(drive->TLA,"N/A\0"); return 1;}
+		{
+			sperror ("Plextor get TLA",drive->err);
+			strcpy(drive->TLA,"N/A\0");
+			return 1;
+		}
 	}
 	memcpy(drive->TLA,drive->rd_buf+0x29,4);
 	drive->TLA[4] = 0;
 	return 0;
 }
 
+int plextor_read_eeprom_CDR(drive_info* drive) {
+//	unsigned int	i,j;
+	drive->cmd[0] = 0xF1;
+	drive->cmd[1] = 0x00;
+
+	drive->cmd[7] = 0x00;
+	drive->cmd[8] = 0x01;
+	drive->cmd[9] = 0x00;
+	drive->cmd[10] = 0x00;
+	drive->cmd[11] = 0x00;
+	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,256) ))
+		{ sperror ("Plextor read CDR EEPROM",drive->err); return drive->err; }
+	return 0;
+}
+
+int plextor_read_eeprom_PX712(drive_info* drive) {
+//	unsigned int	i,j;
+	drive->cmd[0] = 0xF1;
+	drive->cmd[1] = 0x00;
+
+	drive->cmd[7] = 0x00;
+	drive->cmd[8] = 0x02;
+	drive->cmd[9] = 0x00;
+	drive->cmd[10] = 0x00;
+	drive->cmd[11] = 0x00;
+	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,512) ))
+		{ sperror ("Plextor read PX712 EEPROM",drive->err); return drive->err; }
+	return 0;
+}
+
+int plextor_read_eeprom_block(drive_info* drive, unsigned char idx, unsigned int sz) {
+//	unsigned int	i,j;
+	int		offs=idx*sz;
+//	unsigned char*  buf=drive->rd_buf+offs;
+	drive->cmd[0] = 0xF1;
+	drive->cmd[1] = 0x01;
+	drive->cmd[7] = idx;
+	drive->cmd[8] = (sz >> 8) & 0xFF;
+	drive->cmd[9] = sz & 0xFF;
+	drive->cmd[10] = 0x00;
+	drive->cmd[11] = 0x00;
+//	if ((drive->err=drive->cmd.transport(READ,buf,sz) ))
+	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf + offs,sz) ))
+		{ sperror ("Plextor read EEPROM",drive->err); return drive->err; }
+/*
+	printf("EEPROM block #%d:\n",idx);
+	for(i=0;i<(sz/0x10);i++) {
+	    printf("| %X0 | ", i);
+	    for(j=0;j<0x10;j++) printf("%02X ",buf[i*0x10+j]);
+	    printf("|");
+	    for(j=0;j<0x10;j++) {
+		if (buf[i*0x10+j] > 0x20) printf("%c",buf[i*0x10+j]);
+		else printf(" ");
+	    }
+	    printf("|\n");
+	};
+*/
+	return 0;
+}
+
+int plextor_read_eeprom(drive_info* drive, int* len) {
+	int l=0;
+	if (!isPlextor(drive)) {
+		if (len) *len=0;
+		return 1;
+	}
+	switch(drive->dev_ID) {
+		case PLEXTOR_OLD:
+		case PLEXTOR_4824:
+		case PLEXTOR_5224:
+		case PLEXTOR_PREMIUM:
+		case PLEXTOR_PREMIUM2:
+			plextor_read_eeprom_CDR(drive);
+			if ( drive->err ) {
+				if (len) *len=0;
+				return 1;
+			}
+			l = 256;
+			break;
+		case PLEXTOR_708:
+		case PLEXTOR_708A2:
+		case PLEXTOR_712:
+			plextor_read_eeprom_PX712(drive);
+			if ( drive->err ) {
+				if (len) *len=0;
+				return 1;
+			}
+			l = 512;
+			break;
+		case PLEXTOR_714:
+		case PLEXTOR_716:
+		case PLEXTOR_716AL:
+		case PLEXTOR_755:
+		case PLEXTOR_760:
+			for (unsigned char idx=0; idx<4; idx++) {
+				plextor_read_eeprom_block(drive, idx, 256);
+				if ( drive->err ) {
+					if (len) *len=l;
+					return 1;
+				}
+				l+=256;
+			}
+			break;
+		default:
+			printf("Don't know how to read eeprom from this device: %04X:%04X\n", drive->ven_ID, drive->dev_ID);
+			if (len) *len=0;
+			break;
+	}
+	if (len) *len=l;
+	return 0;
+}
+
+int plextor_get_life(drive_info* drive)
+{
+	uint32_t ucr=0;
+	uint32_t ucw=0;
+	uint32_t udr=0;
+	uint32_t udw=0;
+	drive->life.ok=0;
+//	printf("plextor_get_life()\n");
+	if (drive->ven_ID != DEV_PLEXTOR) return 1;
+	if (plextor_read_eeprom(drive)) {
+		printf("Error reading eeprom!\n");
+		return 1;
+	}
+	switch (drive->dev_ID) {
+		case PLEXTOR_OLD:
+		case PLEXTOR_4824:
+		case PLEXTOR_5224:
+		case PLEXTOR_PREMIUM:
+		case PLEXTOR_PREMIUM2:
+			drive->life.dn = ntoh16(drive->rd_buf+0x0078);
+			ucr = ntoh32(drive->rd_buf+0x006C);
+			ucw = ntoh32(drive->rd_buf+0x007A);
+			break;
+		default:
+			drive->life.dn = ntoh16(drive->rd_buf+0x0120);
+			ucr = ntoh32(drive->rd_buf+0x0122);
+			ucw = ntoh32(drive->rd_buf+0x0126);
+
+			udr = ntoh32(drive->rd_buf+0x012A);
+			udw = ntoh32(drive->rd_buf+0x012E);
+			break;
+	}
+	drive->life.ok=1;
+	int2hms(ucr, &drive->life.cr);
+	int2hms(ucw, &drive->life.cw);
+	int2hms(udr, &drive->life.dr);
+	int2hms(udw, &drive->life.dw);
+	return 0;
+}
+
 int plextor_get_speeds(drive_info* drive) {
 	int sel, max, last;
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_PREC_SPD;
 	drive->cmd[9]=0x0A;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x0A) ))
@@ -59,9 +230,9 @@ int plextor_get_speeds(drive_info* drive) {
 //	sel = (drive->rd_buf[5]<<8) | drive->rd_buf[4];
 //	max = (drive->rd_buf[7]<<8) | drive->rd_buf[6];
 //	last = (drive->rd_buf[9]<<8) | drive->rd_buf[8];
-	sel = swap2(drive->rd_buf+4);
-	max = swap2(drive->rd_buf+6);
-	last = swap2(drive->rd_buf+8);
+	sel = ntoh16(drive->rd_buf+4);
+	max = ntoh16(drive->rd_buf+6);
+	last = ntoh16(drive->rd_buf+8);
 	if (!drive->silent) {
 		printf("Selected write speed : %5d kB/s (%d X)\n", sel,  sel/177);
 		printf("Max for this media   : %5d kB/s (%d X)\n", max,  max/177);
@@ -71,7 +242,6 @@ int plextor_get_speeds(drive_info* drive) {
 }
 
 int plextor_get_powerec(drive_info* drive) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE2;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=0x00;
@@ -80,7 +250,7 @@ int plextor_get_powerec(drive_info* drive) {
 		{ if (!drive->silent) sperror ("GET_POWEREC",drive->err); return drive->err; }
 	drive->plextor.powerec_state = drive->rd_buf[2];
 //	drive->plextor.powerec_spd = ((drive->rd_buf[4] & 0xFF)<<8) | (drive->rd_buf[5] & 0xFF);
-	drive->plextor.powerec_spd = swap2(drive->rd_buf+4);
+	drive->plextor.powerec_spd = ntoh16(drive->rd_buf+4);
 	if (!drive->silent) printf("\tPoweRec %s, Recomended speed: %d kB/s\n",
 		drive->plextor.powerec_state ? "ON" : "OFF",drive->plextor.powerec_spd);
 //	printf("\t"); for (int i=0; i<8; i++) printf("0x%02X ",drive->rd_buf[i]&0xFF); printf("\n");
@@ -89,7 +259,6 @@ int plextor_get_powerec(drive_info* drive) {
 
 int plextor_set_powerec(drive_info* drive) {
 	printf("\tTurning PoweRec %s\n",drive->plextor.powerec_state ? "ON" : "OFF");
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE2;
 	drive->cmd[1]=PLEX_SET_MODE | (drive->plextor.powerec_state?1:0);
 	drive->cmd[2]=0x00;
@@ -98,39 +267,85 @@ int plextor_set_powerec(drive_info* drive) {
 		{ if (!drive->silent) sperror ("SET_POWEREC",drive->err);return drive->err;}
 	drive->plextor.powerec_state = drive->rd_buf[2] & 0xFF;
 //	drive->plextor.powerec_spd = ((drive->rd_buf[4] & 0xFF)<<8) | (drive->rd_buf[5] & 0xFF);
-	drive->plextor.powerec_spd = swap2(drive->rd_buf+4);
+	drive->plextor.powerec_spd = ntoh16(drive->rd_buf+4);
 //	printf("\t"); for (int i=0; i<8; i++) printf("0x%02X ",drive->rd_buf[i]&0xFF); printf("\n");
 //	plextor_get_speeds(drive);
 	return 0;
 }
 
-void plextor_print_silentmode_state(drive_info* drive){
+void plextor_print_silentmode_state(drive_info* drive) {
 	int val, idx;
-	val=drive->plextor_silent.read_speed; idx=0;
-	printf("\tRead speed  : ");
-	if ( drive->media.disc_type & DISC_DVD ) {
+	val=drive->plextor_silent.rd; idx=0;
+	printf("Active SilentMode settings:\n");
+	printf("\tSM Read speed  : ");
+	if ( drive->media.type & DISC_DVD ) {
 		while ((silent_dvd_rd_tbl[idx].val!=0xFF) & (silent_dvd_rd_tbl[idx].val!=(val & 0xFF))) idx++;
 		printf("%s\n",silent_dvd_rd_tbl[idx].name);
-	}else{
+//		printf("%s : %d\n",silent_dvd_rd_tbl[idx].name, val);
+	} else {
 		while ((silent_cd_rd_tbl[idx].val!=0xFF) & (silent_cd_rd_tbl[idx].val!=(val & 0xFF))) idx++;
 		printf("%s\n",silent_cd_rd_tbl[idx].name);
+//		printf("%s : %d\n",silent_cd_rd_tbl[idx].name, val);
 	}
-	val=drive->plextor_silent.write_speed; idx=0;
-	printf("\tWrite speed : ");
-	if ( drive->media.disc_type & DISC_DVD ) {
-		while ((silent_dvd_wr_tbl[idx].val!=0xFF) & (silent_dvd_wr_tbl[idx].val!=(val & 0xFF))) idx++;
-		printf("%s\n",silent_dvd_wr_tbl[idx].name);
-	}else{
+	val=drive->plextor_silent.wr; idx=0;
+	printf("\tSM Write speed : ");
+	if ( drive->media.type & DISC_DVD ) {
+//		while ((silent_dvd_wr_tbl[idx].val!=0xFF) & (silent_dvd_wr_tbl[idx].val!=(val & 0xFF))) idx++;
+//		printf("%s : %d\n",silent_dvd_wr_tbl[idx].name, val);
+		printf("MAX\n");
+	} else {
 		while ((silent_cd_wr_tbl[idx].val!=0xFF) & (silent_cd_wr_tbl[idx].val!=(val & 0xFF))) idx++;
 		printf("%s\n",silent_cd_wr_tbl[idx].name);
+//		printf("%s : %d\n",silent_cd_wr_tbl[idx].name, val);
 	}
-	printf("\tAccess time : %s\n",drive->plextor_silent.access_speed?"SLOW":"FAST");
-	printf("\tEject speed : %d\n",drive->plextor_silent.eject_speed);
-	printf("\tLoad speed  : %d\n",drive->plextor_silent.load_speed);
+	printf("\tSM Access time : %s\n",drive->plextor_silent.access?"SLOW":"FAST");
+	printf("\tSM Load speed  : %d\n",drive->plextor_silent.load);
+	printf("\tSM Eject speed : %d\n",drive->plextor_silent.eject);
+
+	if (!drive->plextor_silent.psaved) {
+		printf("Saved SilentMode settings not found\n");
+		return;
+	}
+
+	printf("Saved SilentMode settings:\n");
+
+	printf("\tPSM Silent State   : %s\n",drive->plextor_silent.pstate?"ON":"OFF");
+
+	printf("\tPSM CD Read speed  : %dX\n",drive->plextor_silent.prd_cd);
+	printf("\tPSM CD Write speed : %dX\n",drive->plextor_silent.pwr_cd);
+	printf("\tPSM DVD Read speed : %dX\n",drive->plextor_silent.prd_dvd);
+//	printf("\tPSM DVD Write speed: %d\n",drive->plextor_silent.pwr_dvd);
+
+	printf("\tPSM Access time : %s\n",drive->plextor_silent.paccess?"SLOW":"FAST");
+	printf("\tPSM Load speed  : %d\n",drive->plextor_silent.pload);
+	printf("\tPSM Eject speed : %d\n",drive->plextor_silent.peject);
+}
+
+void plextor_get_silentmode_saved(drive_info* drive) {
+	int len;
+	if (plextor_read_eeprom(drive, &len) || len<0x110) return;
+
+	drive->plextor_silent.psaved  = 1;
+	drive->plextor_silent.pstate  = (drive->rd_buf[0x100] == 1);
+	drive->plextor_silent.paccess = (drive->rd_buf[0x101] == 2);
+
+	drive->plextor_silent.prd_cd  = drive->rd_buf[0x102];
+	drive->plextor_silent.prd_dvd = drive->rd_buf[0x103];
+	drive->plextor_silent.pwr_cd  = drive->rd_buf[0x104];
+//	drive->plextor_silent.pwr_dvd = 16;
+
+//								= drive->rd_buf[0x106]; // ???
+
+/*
+	drive->plextor_silent.pload   = drive->rd_buf[0x108] - 47;
+	drive->plextor_silent.peject  = 48 - drive->rd_buf[0x107];
+*/
+	drive->plextor_silent.pload   = drive->rd_buf[0x108] - 47;
+	drive->plextor_silent.peject  = -(drive->rd_buf[0x107]+48);
 }
 
 int plextor_get_silentmode(drive_info* drive) {
-	drive->cmd_clear();
+	drive->plextor_silent.psaved = 0;
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SILENT;
@@ -141,17 +356,17 @@ int plextor_get_silentmode(drive_info* drive) {
 	if (( drive->err=drive->cmd.transport(READ,(void*)&(drive->plextor_silent),8) ))
 		{ if (!drive->silent) sperror ("GET_SILENT_MODE",drive->err); return drive->err; }
 //	printf("\t"); for (int i=0; i<8; i++) printf("0x%02X ",drive->rd_buf[i]&0xFF); printf("\n");
+	plextor_get_silentmode_saved(drive);
 	return 0;
 }
 
 int plextor_set_silentmode_tray(drive_info* drive, int disc_type, int permanent) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SILENT_TRAY;
 	drive->cmd[3]=disc_type | 2*!!permanent;
-	drive->cmd[4]=drive->plextor_silent.eject_speed;
-	drive->cmd[6]=drive->plextor_silent.load_speed;
+	drive->cmd[4]=drive->plextor_silent.eject;
+	drive->cmd[6]=drive->plextor_silent.load;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0)))
 		{ if (!drive->silent) sperror ("SET_SILENT_MODE_DISC",drive->err);return drive->err;}
 //	printf("\t"); for (int i=0; i<8; i++) printf("0x%02X ",drive->rd_buf[i]&0xFF); printf("\n");
@@ -159,14 +374,17 @@ int plextor_set_silentmode_tray(drive_info* drive, int disc_type, int permanent)
 }
 
 int plextor_set_silentmode_disc(drive_info* drive, int disc_type, int permanent) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SILENT_DISC;
 	drive->cmd[3]=disc_type | 2*!!permanent;
-	drive->cmd[4]=drive->plextor_silent.read_speed;
-	drive->cmd[5]=drive->plextor_silent.write_speed;
-	drive->cmd[6]=drive->plextor_silent.access_speed;
+	drive->cmd[4]=drive->plextor_silent.rd;
+	if (disc_type == SILENT_CD) {
+		drive->cmd[5]=drive->plextor_silent.wr;
+	} else {
+		drive->cmd[5]=0xFF;
+	}
+	drive->cmd[6]=drive->plextor_silent.access;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0)))
 		{ if (!drive->silent) sperror ("SET_SILENT_MODE_DISC",drive->err);return drive->err;}
 //	printf("\t"); for (int i=0; i<8; i++) printf("0x%02X ",drive->rd_buf[i]&0xFF); printf("\n");
@@ -174,13 +392,12 @@ int plextor_set_silentmode_disc(drive_info* drive, int disc_type, int permanent)
 }
 
 int plextor_set_silentmode_disable(drive_info* drive, int permanent) {
-
-	drive->plextor_silent.read_speed=0x07;
-	drive->plextor_silent.write_speed=0x07;
-	drive->plextor_silent.access_speed=0;
+	drive->plextor_silent.rd=0x07;
+	drive->plextor_silent.wr=0x07;
+	drive->plextor_silent.access=0;
 	plextor_set_silentmode_disc(drive, 0, permanent);
-	drive->plextor_silent.eject_speed=0x50;
-	drive->plextor_silent.load_speed=0x50;
+	drive->plextor_silent.eject=0x50;
+	drive->plextor_silent.load=0x50;
 	plextor_set_silentmode_tray(drive, 0, permanent);
 	return 0;
 }
@@ -200,7 +417,6 @@ void print_gigarec_value(drive_info* drive) {
 int plextor_set_gigarec(drive_info* drive) {
 // 	printf("  applying gigarec setting... ");
 // 	print_gigarec_value(i);
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_GIGAREC;
@@ -217,7 +433,6 @@ int plextor_set_gigarec(drive_info* drive) {
 }
 
 int plextor_get_gigarec(drive_info* drive) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_GIGAREC;
@@ -235,18 +450,21 @@ void print_varirec(drive_info* drive, int disc_type) {
 	if (disc_type == VARIREC_DVD) {
 		i=drive->plextor.varirec_pwr_dvd;
 		s=drive->plextor.varirec_str_dvd;
-	}else{
+	} else {
 		i=drive->plextor.varirec_pwr_cd;
 		s=drive->plextor.varirec_str_cd;
 	}
 	v=0;
 	while ((varirec_pwr_tbl[v].val!=0xFF) & (varirec_pwr_tbl[v].val!=(i & 0xFF))) v++;
-	printf("\t%s power   : %s\n", (disc_type == VARIREC_DVD)? "DVD":"CD ", varirec_pwr_tbl[v].name);
-	printf("\tStrategy    : %s\n", (disc_type == VARIREC_DVD)? varirec_str_dvd_tbl[s] : varirec_str_cd_tbl[s]);
+	printf("\tVariRec %s power    : %s\n",
+									(disc_type == VARIREC_DVD)? "DVD":"CD ",
+									varirec_pwr_tbl[v].name);
+	printf("\tVariRec %s strategy : %s [%d]\n",
+									(disc_type == VARIREC_DVD)? "DVD":"CD ",
+									(disc_type == VARIREC_DVD)? varirec_str_dvd_tbl[s] : varirec_str_cd_tbl[s], s);
 }
 
 int plextor_set_varirec(drive_info* drive, int disc_type) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_VARIREC;
@@ -276,7 +494,6 @@ int plextor_set_varirec(drive_info* drive, int disc_type) {
 }
 
 int plextor_get_varirec(drive_info* drive, int disc_type) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_VARIREC;
@@ -306,26 +523,26 @@ void print_securec_state(drive_info* drive) {
 
 int plextor_get_securec_state(drive_info* drive)
 {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[2]=PLEX_MODE_SECUREC;
 	drive->cmd[10]=0x08;
-	if (drive->err = drive->cmd.transport(READ, drive->rd_buf, 8))
+	if ((drive->err = drive->cmd.transport(READ, drive->rd_buf, 8)))
 		{ if (!drive->silent) sperror("PLEXTOR_GET_SECUREC", drive->err); return drive->err;}
 	drive->plextor.securec = drive->rd_buf[3];
 	drive->plextor.securec_disc = drive->rd_buf[4];
 //	drive->plextor.gigarec = (drive->rd_buf[3] & 0xFF);
 //	drive->plextor.gigarec_disc = (drive->rd_buf[4] & 0xFF);
 
+#ifdef DEBUG_SECUREC
 	printf("get_securec() data: ");
 	for (int i=0; i<8; i++) printf("0x%02X ",drive->rd_buf[i]&0xFF); printf("\n");
+#endif // DEBUG_SECUREC
 	return 0;
 }
 
 int plextor_set_securec(drive_info* drive, char len, char* passwd)
 {
-	char i;
-	drive->cmd_clear();
+	int i;
 	drive->cmd[0]=PLEXTOR_SEND_AUTH;
 	if ((passwd) && (len)) {
 		printf("Turning SecuRec ON\n");
@@ -340,14 +557,14 @@ int plextor_set_securec(drive_info* drive, char len, char* passwd)
 		for(i=0;i<0x0E;i++)
 			if (i<len) drive->rd_buf[i+2]=passwd[i];
 			else drive->rd_buf[i+2]=0;
-		if (drive->err = drive->cmd.transport(WRITE, drive->rd_buf, 0x10))
+		if ((drive->err = drive->cmd.transport(WRITE, drive->rd_buf, 0x10)))
 			{if (!drive->silent) sperror("PLEXTOR_SET_SECUREC", drive->err); return drive->err;}
 	}else{
 		printf("Turning SecuRec OFF\n");
-		if (drive->err = drive->cmd.transport(NONE, NULL, 0))
+		if ((drive->err = drive->cmd.transport(NONE, NULL, 0)))
 			{if (!drive->silent) sperror("PLEXTOR_SET_SECUREC", drive->err); return drive->err;}
 	}
-	if ((drive->dev_ID == PLEXTOR_760)) px755_do_auth(drive);
+//	if ((drive->dev_ID == PLEXTOR_760)) plextor_px755_do_auth(drive);
 	return 0;
 }
 
@@ -356,7 +573,6 @@ void print_speedread_state(drive_info* drive){
 }
 
 int plextor_set_speedread(drive_info* drive, int state) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_SPDREAD;
@@ -371,7 +587,6 @@ int plextor_set_speedread(drive_info* drive, int state) {
 
 int plextor_get_speedread(drive_info* drive) {
 //	return 1;
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SPDREAD;
@@ -392,7 +607,6 @@ void print_sss_state(drive_info* drive) {
 }
 
 int plextor_get_hidecdr_singlesession(drive_info* drive) {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SS_HIDE;
@@ -407,7 +621,6 @@ int plextor_get_hidecdr_singlesession(drive_info* drive) {
 int plextor_set_hidecdr_singlesession(drive_info* drive, int hidecdr_state, int singlesession_state) {
 	if (plextor_get_hidecdr_singlesession(drive)) return 1;
 //	printf("Trying to change SS/HIDE state to %d...\n",2*!!hidecdr_state + !!singlesession_state);
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_SS_HIDE;
@@ -435,7 +648,6 @@ int plextor_set_singlesession(drive_info* drive, int state) {
 
 int plextor_get_bitset(drive_info* drive, int disc_type)
 {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_BITSET;
@@ -460,7 +672,6 @@ int plextor_set_bitset(drive_info* drive, int disc_type)
 		case PLEX_BITSET_RDL:	book = (drive->book_plus_rdl); break;
 		default:		printf("PLEXTOR_SET_BITSET: Invalid disc_type"); return 1;
 	}
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_BITSET;
@@ -474,7 +685,6 @@ int plextor_set_bitset(drive_info* drive, int disc_type)
 
 int plextor_get_testwrite_dvdplus(drive_info* drive)
 {
-	drive->cmd_clear();
 	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_TESTWRITE_DVDPLUS;
@@ -487,7 +697,6 @@ int plextor_get_testwrite_dvdplus(drive_info* drive)
 
 int plextor_set_testwrite_dvdplus(drive_info* drive)
 {
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_MODE;
 	drive->cmd[1] = PLEX_SET_MODE;
 	drive->cmd[2] = PLEX_MODE_TESTWRITE_DVDPLUS;
@@ -502,14 +711,13 @@ int plextor_plexeraser(drive_info* drive)
 	long i;
 	printf("Destucting disc [mode=%02X]... \n",drive->plextor.plexeraser);
 //	return 0;
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_PLEXERASER;
 	drive->cmd[1] = 0x06;
 	drive->cmd[2] = drive->plextor.plexeraser;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0) ))
 		{ if (!drive->silent) sperror ("PLEXTOR_DO_PLEXERASER",drive->err); return drive->err;}
 	while (test_unit_ready(drive)) {
-		usleep(1000000);
+		msleep(1000);
 		i++;
 	}
 	return 0;
@@ -521,7 +729,7 @@ int plextor_plexeraser(drive_info* drive)
 
 int plextor_print_autostrategy_state(drive_info* drive)
 {
-	printf("AutoStrategy        : ");
+	printf("AutoStrategy mode : ");
 	switch (drive->astrategy.state)
 	{
 		case AS_OFF: printf("OFF"); break;
@@ -536,7 +744,6 @@ int plextor_print_autostrategy_state(drive_info* drive)
 
 int plextor_get_autostrategy(drive_info* drive)
 {
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[10]= 0x08;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,8) ))
@@ -548,7 +755,6 @@ int plextor_get_autostrategy(drive_info* drive)
 
 int plextor_set_autostrategy(drive_info* drive)
 {
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[2] = PLEX_SET_MODE + (drive->astrategy.state & 0x0F);
 	drive->cmd[10]= 0x08;
@@ -562,7 +768,6 @@ int plextor_set_autostrategy(drive_info* drive)
 
 int plextor_get_autostrategy_db_entry_count(drive_info* drive)
 {
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x02;
 	drive->cmd[10]= 0x08;
@@ -577,7 +782,6 @@ int plextor_get_autostrategy_db(drive_info* drive)//, void* database)
 {
 	int size = 8 + (int)drive->astrategy.dbcnt * 32;
 	int i,j;
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD; // 0xE4
 	drive->cmd[1] = 0x02;
 	drive->cmd[9]= (size >> 8) & 0xFF ;
@@ -587,14 +791,14 @@ int plextor_get_autostrategy_db(drive_info* drive)//, void* database)
 	printf("** AS DB entries: %d\n",drive->astrategy.dbcnt);
 //*
 	for (j=0; j<drive->astrategy.dbcnt; j++) {
-		printf("S#%02d: ", drive->astrategy.entry[j].number);
-		printf("[%c]",drive->astrategy.entry[j].enabled ? '*':' ');
-		printf("DVD%cR [%02X] |", (drive->astrategy.entry[j].type == 0xA1)? '+':'-',drive->astrategy.entry[j].type);
-		printf("%3dX |",drive->astrategy.entry[j].speed);
-		for (i=0; i<12;i++)
-			if (drive->astrategy.entry[j].MID[i] < 0x20) drive->astrategy.entry[j].MID[i] = 0x20;
-		printf("%13s |",drive->astrategy.entry[j].MID);
-		printf("%d\n",(drive->astrategy.entry[j].counter[0] <<8) | drive->astrategy.entry[j].counter[1]);
+		for (i=0; i<12;i++) if (drive->astrategy.entry[j].MID[i] < 0x20) drive->astrategy.entry[j].MID[i] = 0x20;
+		printf("S#%02d |%c| DVD%cR [%02X] | %3dX | %13s | %d\n",
+				drive->astrategy.entry[j].number,
+				drive->astrategy.entry[j].enabled ? '*':' ',
+				(drive->astrategy.entry[j].type == 0xA1)? '+':'-', drive->astrategy.entry[j].type,
+				drive->astrategy.entry[j].speed,
+				drive->astrategy.entry[j].MID,
+				(drive->astrategy.entry[j].counter[0] <<8) | drive->astrategy.entry[j].counter[1]);
 	}
 //*/
 /*
@@ -629,7 +833,6 @@ int plextor_get_strategy(drive_info* drive){
 	unsigned char *entry_data;
 
 	printf("RETR AS cnt...\n");
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD; // 0xE4
 	drive->cmd[1] = 0x02;
 	drive->cmd[2] = 0x03;
@@ -643,7 +846,6 @@ int plextor_get_strategy(drive_info* drive){
 	printf("\nStrategies count: %d\n", drive->astrategy.dbcnt);
 
 	printf("RETR AS data...\n");
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD; // 0xE4
 	drive->cmd[1] = 0x02;
 	drive->cmd[2] = 0x03;
@@ -709,7 +911,7 @@ int plextor_add_strategy(drive_info* drive){
 
 	entry = (unsigned char*) &drive->astrategy.entry;
 	for (i=0; i<cnt*0x20; i++) drive->rd_buf[8+i] = entry[i];
-	for (i=0; i<cnt; i++) drive->rd_buf[8+i*0x20] = (unsigned char)(i+1 & 0xFF);
+	for (i=0; i<cnt; i++) drive->rd_buf[8+i*0x20] = (unsigned char)((i+1) & 0xFF);
 
 #ifdef ASDB_LOAD_DEBUG
 	printf("DB HDR0:\n");
@@ -721,7 +923,6 @@ int plextor_add_strategy(drive_info* drive){
 	}
 #endif
 
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
 	drive->cmd[1] = 0x02;
 	drive->cmd[9] = (cnt >> 3) & 0xFF;
@@ -762,7 +963,6 @@ int plextor_add_strategy(drive_info* drive){
 	}
 #endif
 
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
 	drive->cmd[1] = 0x02;
 	drive->cmd[9] = ((cnt*0xE) >> 4) & 0xFF;
@@ -777,7 +977,6 @@ int plextor_clear_autostrategy_db(drive_info* drive) {
 	drive->rd_buf[1]=0x06;
 	drive->rd_buf[2]=0x02;
 	drive->rd_buf[3]=0xFF;
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
 	drive->cmd[1] = 0x02;
 	drive->cmd[10]= 0x08;
@@ -800,7 +999,6 @@ int plextor_modify_autostrategy_db(drive_info* drive, int index, int action)
 	drive->rd_buf[8]=index;
 	drive->rd_buf[9]=action;
 
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
 	drive->cmd[1] = 0x02;
 	drive->cmd[10]= 0x0A;
@@ -811,9 +1009,9 @@ int plextor_modify_autostrategy_db(drive_info* drive, int index, int action)
 
 int plextor_create_strategy(drive_info* drive, int mode)
 {
-	int i;
+	int i=0;
+	int p=0;
 //	if (!drive->silent) printf("AS create: %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X\n",		(CMD_PLEX_AS_RD) & 0xFF, 4, mode & 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x04;
 	drive->cmd[2] = mode;
@@ -822,27 +1020,27 @@ int plextor_create_strategy(drive_info* drive, int mode)
 
 	if (!drive->silent) printf("AS CRE START...\n");
 
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[10]= 0x12;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
-		{ if (!drive->silent) sperror ("PLEXTOR_CREATE_STRATEGY",drive->err); return drive->err;}
+		{ if (!drive->silent) sperror ("PLEXTOR_AS_GET_STATUS",drive->err); return drive->err;}
 
 	if (!drive->silent) printf("      AS CRE: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i] & 0x0FF); printf("\n");
+
 // Waiting until Strategy is created
 	while (test_unit_ready(drive)) {
 		sleep(1);
+		printf("%c\r", progress[p++]); if (p==4) p=0;
 		i++;
 	}
 	printf("Strategy creation time: %d sec\n",i);
 
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[10]= 0x12;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
-		{ if (!drive->silent) sperror ("PLEXTOR_CREATE_STRATEGY_DONE",drive->err); return drive->err;}
+		{ if (!drive->silent) sperror ("PLEXTOR_AS_GET_STATUS",drive->err); return drive->err;}
 
 //	if (!drive->silent) printf(" AS CRE DONE: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i]) & 0x0FF; printf("\n");
 
@@ -850,114 +1048,41 @@ int plextor_create_strategy(drive_info* drive, int mode)
 }
 
 int plextor_media_check(drive_info* drive, int mode)
-//int plextor_create_strategy(drive_info* drive, int mode)
 {
 	int i=0;
-	if (!drive->silent) printf("MQCK: %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X\n",
-		(PLEXTOR_AS_RD) & 0xFF, 1, mode & 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-//	return 0;
+	int p=0;
+//	if (!drive->silent) printf("MQCK CDB: %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X\n",
+//		(PLEXTOR_AS_RD) & 0xFF, 1, mode & 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+
+	if ( !(drive->media.type & (DISC_DVD)) ) {
+		printf("Media Quality Check supported on DVD media only!\n");
+		return -1;
+	}
 
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[2] = mode;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0)))
 		{ if (!drive->silent) sperror ("PLEXTOR_MEDIA_QUALITY_CHECK_START",drive->err); return drive->err;}
 
-	printf("MQCK START...\n");
-/*
-	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
-	drive->cmd[1] = 0x01;
-	drive->cmd[10]= 0x12;
-	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
-		{ sperror ("PLEXTOR_CREATE_STRATEGY",drive->err); return drive->err;}
+	printf("Starting MQCK...\n");
 
-	printf("      AS CRE: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i] & 0xFF); printf("\n");
-*/
 // Waiting until Media is checked
 	while (test_unit_ready(drive)) {
 		sleep(1);
+		printf("%c\r", progress[p++]); if (p==4) p=0;
 		i++;
 	}
-	printf("Media check time: %d sec",i);
+	printf("\nMedia Check time: %d sec", i);
 
-	drive->cmd_clear();
 	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[10]= 0x12;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
-		{ if (!drive->silent) sperror ("PLEXTOR_MEDIA_QUALITY_CHECK_DONE",drive->err); return drive->err;}
-
-	printf(" MQCK DONE: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i]) & 0xFF; printf("\n");
+		{ if (!drive->silent) sperror ("PLEXTOR_AS_GET_STATUS",drive->err); return drive->err; }
 
+	if(!drive->silent)
+		{ printf("Media Check result RAW: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i]); printf("\n"); }
 	return 0;
 }
 
-		//----------------//
-		//  PX-755 AUTH   //
-		//----------------//
-
-int px755_do_auth(drive_info* drive)
-{
-	if ((drive->dev_ID < PLEXTOR_760))
-		{ if (!drive->silent) printf("Plextor drive is older than PX-755, auth not needed\n"); return 0; }
-	px755_clear_auth_status(drive);
-	px755_get_auth_code(drive, drive->rd_buf);
-	px755_calc_auth_code(drive->rd_buf);
-	if (px755_send_auth_code(drive, drive->rd_buf)) {
-		printf(" _______________________________________________________ \n");
-		printf("|                                                       |\n");
-		printf("|       WARNING!!!  Detected locked PX-755/PX-760!      |\n");
-		printf("|              It has protected commands                |\n");
-		printf("|    you'll not get full fucntionality of this drive    |\n");
-		printf("|_______________________________________________________|\n");
-		return 1;
-	} else {
-		if (!drive->silent) printf("PX-755/760 auth successfull:)\n");
-		return 0;
-	}
-}
-
-int px755_get_auth_code(drive_info* drive,unsigned char* auth_code)
-{
-	drive->cmd_clear();
-	drive->cmd[0] = PLEXTOR_GET_AUTH;
-	drive->cmd[10]= 0x10;
-	if ((drive->err=drive->cmd.transport(READ,auth_code,16) ))
-		{ if (!drive->silent) sperror ("PLEXTOR_PX755_GET_AUTH_CODE",drive->err); return drive->err;}
-	if (!drive->silent) {
-		printf("** Get PX755 auth: ");
-		for (int i=0; i<16; i++) printf("0x%02X ",drive->rd_buf[i]&0xFF); printf("\n");
-	}
-	return 0;
-}
-
-int px755_send_auth_code(drive_info* drive, unsigned char* auth_code)
-{
-	drive->cmd_clear();
-	drive->cmd[0] = PLEXTOR_SEND_AUTH;
-	drive->cmd[1] = 0x01;
-	drive->cmd[2] = 0x01;
-	drive->cmd[10]= 0x10;
-	if ((drive->err=drive->cmd.transport(WRITE,auth_code,16) ))
-		{ if (!drive->silent) sperror ("PLEXTOR_PX755_SEND_AUTH_CODE",drive->err); return drive->err;}
-	return 0;
-}
-
-int px755_clear_auth_status(drive_info* drive)
-{
-	drive->cmd_clear();
-	drive->cmd[0] = PLEXTOR_SEND_AUTH;
-	drive->cmd[1] = 0x01;
-	drive->cmd[2] = 0x00;
-	drive->cmd[10]= 0x10;
-	if ((drive->err=drive->cmd.transport(NONE,NULL,0) ))
-		{ if (!drive->silent) sperror ("PLEXTOR_PX755_CLEAR_AUTH_STATUS",drive->err); return drive->err;}
-	return 0;
-}
-
-int px755_calc_auth_code(unsigned char* auth_code)
-{
-	return 0;
-}
